unit fpwmmanager;

{$mode objfpc}{$H+}

interface

{$include fpwmdefines.inc}

uses
  // RTL, FCL
  SysUtils, fgl, Classes,
  // UNIX
  ctypes, x, xlib, xutil, baseunix, unix, xshape, xproto, xatom,
  {$ifdef FPWM_USE_LCL}
  // LCL
  Menus,
  {$endif}
  //fpwm
  {$ifdef FPWM_ROTATED}
  fpwmrotated,
  {$endif}
  fpwmgeneral;

// From Manager.h

{#include "General.h"
#include "listmacro2.h"

class Client;
declarePList(ClientList, Client);}

// From Client.h

{#include "General.h"
#include "Manager.h"
#include "Border.h"}

const
  Pdelete    = 1;
  PtakeFocus = 2;

type
  TFPWMBorder = class;
  TFPWMClient = class;

  TClientList = specialize TFPGList<TFPWMClient>;

  RootCursor = (NormalCursor, DeleteCursor, DownCursor, RightCursor, DownrightCursor);

  TFPWMWindowManager = class
  private
    m_display: PDisplay;
    m_screenNumber: cint;

    m_root: TWindow;

    m_defaultColormap: TColormap;
    m_minimumColormaps: cint;

    m_cursor: TCursor;
    m_xCursor: TCursor;
    m_vCursor: TCursor;
    m_hCursor: TCursor;
    m_vhCursor: TCursor;

    m_terminal: PChar;
    m_shell: PChar;

    m_clients: TClientList;
    m_hiddenClients: TClientList;
    m_activeClient: TFPWMClient;

    m_shapeEvent: cint;
    m_currentTime: cint;

    m_looping: Boolean;
    m_returnCode: cint;

    procedure release();

    class function makeCursor(d: PDisplay; w: TWindow;
      bits, mask_bits: PByte; width, height, xhot, yhot: cint;
      var fg, bg: TXColor): TCursor;
    class procedure sigHandler();

    procedure initialiseScreen();
    procedure scanInitialWindows();
  private
    m_menuGC: TGC;
    m_menuWindow: TWindow;
    m_menuFont: PXFontStruct;
    m_menuForegroundPixel: culong;
    m_menuBackgroundPixel: culong;
    m_menuBorderPixel: culong;
//    function menuLabel(p1: cint): PChar;
    procedure showDesktopMenu(e: PXButtonEvent);
    {$ifdef FPWM_USE_LCL}
    procedure showDesktopMenuLCL(e: PXButtonEvent);
    {$else}
    procedure showDesktopMenuX11(e: PXButtonEvent);
    {$endif}
    procedure spawn();
    procedure circulate(activeFirst: Boolean);
  private
    m_focusChanging: Boolean;	// checking times for focus change
    m_focusCandidate: TFPWMClient;
    m_focusCandidateWindow: TWindow;
    m_focusTimestamp: X.TTime;	// time of last crossing event
    m_focusPointerMoved: Boolean;
    m_focusPointerNowStill: Boolean;
    procedure checkDelaysForFocus();

    procedure nextEvent(e: PXEvent);	// return

    procedure eventButton(e: PXButtonEvent);
    procedure eventMapRequest(e: PXMapRequestEvent);
    procedure eventConfigureRequest(e: PXConfigureRequestEvent);
    procedure eventUnmap(e: PXUnmapEvent);
    procedure eventCreate(e: PXCreateWindowEvent);
    procedure eventDestroy(e: PXDestroyWindowEvent);
    procedure eventClient(e: PXClientMessageEvent);
    procedure eventColormap(e: PXColormapEvent);
    procedure eventProperty(e: PXPropertyEvent);
    procedure eventEnter(e: PXCrossingEvent);
    procedure eventReparent(e: PXReparentEvent);
    procedure eventFocusIn(e: PXFocusInEvent);
  public
    constructor Create();
    destructor Destroy();

    function MainLoop(): cint; // was: loop()
    procedure InitWM();

    procedure fatal(const message: string);
  public
    // for call from Client and within:

    function windowToClient(w: TWindow; acreate: Boolean = False): TFPWMClient;
    function activeClient(): TFPWMClient;// { return m_activeClient; }
    function raiseTransients(C: TFPWMClient): Boolean; // true if raised any
    function timestamp(reset: Boolean): x.TTime;
    procedure clearFocus();

    procedure setActiveClient(const c: TFPWMClient);// { m_activeClient = c; }

    procedure addToHiddenList(c: TFPWMClient);
    procedure removeFromHiddenList(c: TFPWMClient);
    procedure skipInRevert(c, myRevert: TFPWMClient);

    function display(): PDisplay;// { return m_display; }
    function root(): TWindow;// { return m_root; }

    procedure installCursor(c: RootCursor);
    procedure installCursorOnWindow(c: RootCursor; w: TWindow);
    procedure installColormap(cmap: TColormap);
    function allocateColour(name, desc: string): culong;

    procedure considerFocusChange(c: TFPWMClient; w: TWindow; atimestamp: x.TTime);
    procedure stopConsideringFocus();

    // shouldn't really be public
    function attemptGrab(w: TWindow; constrain: TWindow; mask: Integer; t: Integer): Integer;
    procedure releaseGrab(e: PXButtonEvent);
    procedure eventExposure(e: PXExposeEvent);	// for exposures during client grab
    procedure showGeometry(x, y: cint);
    procedure removeGeometry();
  end;

  TFPWMClient = class
  public
    constructor Create(wm: TFPWMWindowManager; w: TWindow);
    procedure release();

    { for call from WindowManager: }

    procedure activate();		{ active() }
    procedure deactivate();		{ setactive(0) }
    procedure gravitate(invert: Boolean);
    procedure installColormap();
    procedure unreparent();
    procedure withdraw(changeState: Boolean = True);
    procedure hide();
    procedure unhide(map: Boolean);
    procedure rename();
    procedure kill();
    procedure mapRaised();		// without activating
    procedure lower();

    procedure move(e: PXButtonEvent);		// event for grab timestamp & coords
    procedure resize(e: PXButtonEvent; horizontal, vertical: Boolean);
    procedure moveOrResize(e: PXButtonEvent);
    procedure ensureVisible();	// make sure x, y are on-screen

    procedure manage(mapped: Boolean);
    function hasWindow(w: TWindow): Boolean; {
	return ((m_window == w) || m_border->hasWindow(w));
    }

    function revertTo(): TFPWMClient; { return m_revert; }
    procedure setRevertTo(c: TFPWMClient); { m_revert = c; }

    function isHidden(): Boolean;     { return (m_state == IconicState);    }
    function isWithdrawn(): Boolean;  { return (m_state == WithdrawnState); }
    function isNormal(): Boolean;     { return (m_state == NormalState);    }
    function isTransient(): Boolean;  { return (m_transient != None);       }
    function transientFor(): TWindow; { return m_transient; }
    function isFixedSize(): Boolean;  { return m_fixedSize; }

    function GetLabel(): string;    { return m_label;    }
    function name(): string;     { return m_name;     }
    function iconName(): string; { return m_iconName; }

    procedure sendMessage(a: TAtom; l: clong);
    procedure sendConfigureNotify();

    procedure activateAndWarp();
    procedure focusIfAppropriate(ifActive: Boolean);
    procedure selectOnMotion(w: TWindow; select: Boolean);

    { for call from within: }

    procedure fatal(m: string);    { m_windowManager->fatal(m);              }
    function display(): PDisplay;     { return m_windowManager->display();      }
    function parent(): TWindow;        { return m_border->parent();              }
    function root(): TWindow;          { return m_windowManager->root();         }
    function activeClient(): TFPWMClient; { return m_windowManager->activeClient(); }
    function isActive(): Boolean;     { return (activeClient() == this);        }

    function windowManager(): TFPWMWindowManager; { return m_windowManager; }

    // for call from equivalent wm functions in Events.C:

    procedure eventButton(e: PXButtonEvent);
    procedure eventMapRequest(e: PXMapRequestEvent);
    procedure eventConfigureRequest(e: PXConfigureRequestEvent);
    procedure eventUnmap(e: PXUnmapEvent);
    procedure eventColormap(e: PXColormapEvent);
    procedure eventProperty(e: PXPropertyEvent);
    procedure eventEnter(e: PXCrossingEvent);
    procedure eventFocusIn(e: PXFocusInEvent);
    procedure eventExposure(e: PXExposeEvent);
  public
    destructor Destroy();
  private
    m_window: TWindow;
    m_transient: TWindow;
    m_border: TFPWMBorder;

    m_revert: TFPWMClient;

    m_x, m_y, m_w, m_h, m_bw: cint;

    m_sizeHints: TXSizeHints;
    m_fixedSize: Boolean;
    m_minWidth: cint;
    m_minHeight: cint;
    procedure fixResizeDimensions(var w, h, dw, dh: cint);
    function coordsInHole(x, y: cint): Boolean;
  private
    m_state: cint;
    m_protocol: cint;
    m_managed: Boolean;
    m_reparenting: Boolean;
    m_stubborn: Boolean;		// keeps popping itself to the front
    m_lastPopTime: x.TTime; // Time

    m_name: string;
    m_iconName: string;
    m_label: string;	// alias: one of (instance,class,name,iconName)

    m_colormap: TColormap;
    m_colormapWinCount: cint;
    m_colormapWindows: PWindow;
    m_windowColormaps: PColormap;

    m_windowManager: TFPWMWindowManager;

    function getProperty(a: TAtom): string;
    function getAtomProperty(a, type_: TAtom): cint;
    function getIntegerProperty(a: TAtom): cint;
  private
    // accessors 
    function getState(var state: cint): Boolean;
    procedure setState(state: cint);
  private
    // internal instantiation requests
    function setLabel(): Boolean;	// returns True if changed
    procedure getColormaps();
    procedure getProtocols();
    procedure getTransient();

    procedure decorate(active: Boolean);
  end;

  TFPWMBorder = class  // friend of client
  private
    m_client: TFPWMClient;

    m_parent, m_tab, m_child,
     m_button, m_resize: TWindow;

    m_label: string;

    procedure fatal(s: string);

    procedure fixTabHeight(maxHeight: cint);
    procedure drawLabel();

    procedure setFrameVisibility(visible: Boolean; w, h: cint);
    procedure setTransientFrameVisibility(visible: Boolean; w, h: cint);
    procedure shapeParent(w, h: cint);
    procedure shapeTransientParent(w, h: cint);
    procedure shapeTab(w, h: cint);
    procedure resizeTab(h: cint);	// for rename without changing window size
    procedure shapeResize();
  private
    m_prevW: cint;
    m_prevH: cint;

    m_tabHeight: cint;	// depends on the label

    constructor Create(const c: TFPWMClient; child: TWindow);
    destructor Destroy();

    procedure map();
    procedure unmap();
    procedure lower();
    procedure mapRaised();
    procedure decorate(active: Boolean; w, h: cint);
    procedure reparent();
    procedure configure(x, y, w, h: cint; mask: culong; detail: cint;
		   force: Boolean = False);
    procedure moveTo(x, y: cint);

    function windowManager(): TFPWMWindowManager; // calls into Client
    function isTransient(): Boolean;	// calls into Client
    function isFixedSize(): Boolean;	// calls into Client}
    function parent(): TWindow; { return m_parent; }
    function hasWindow(w: TWindow): Boolean; {
	//return (w != root() && (w == m_parent || w == m_tab ||
				//w == m_button || w == m_resize));
    //}

    function display(): PDisplay;
    function root(): TWindow;

    procedure expose_(e: PXExposeEvent);
    procedure eventButton(e: PXButtonEvent); // in Buttons.C

    function yIndent(): cint;// {
//	return isTransient() ? TRANSIENT_FRAME_WIDTH + 1 : FRAME_WIDTH + 1;
    //}
    function xIndent(): cint;// {
	//return isTransient() ? TRANSIENT_FRAME_WIDTH + 1 :
	    //m_tabWidth + FRAME_WIDTH + 1;
    //}

    function coordsInHole(x, y: cint): Boolean; // in Events.C of all places
  end;

var
  m_initialising: Boolean;
  m_defaultLabel: string = 'incognito';

  // from TFPWMWindowManager
  m_menuCreateLabel: string = 'New';

  // from TFPWMClient
  m_signalled: cint;

  // from border.h
  m_tabHeight: cint = -1;  // depends on the label font
  m_tabWidth: cint = -1;
  {$ifdef FPWM_ROTATED}
  m_tabFont: PXRotFontStruct = nil;
  {$else}
  m_tabFont: TXFontStruct;
  {$endif}
  m_drawGC: xlib.TGC = nil;
  m_foregroundPixel,
   m_backgroundPixel,
   m_frameBackgroundPixel,
   m_buttonBackgroundPixel,
   m_borderPixel: culong;
  borderCounter: cint = 0;


function errorHandler(d: PDisplay; e: PXErrorEvent): cint; cdecl;
function nobuttons(e: PXButtonEvent): Boolean; // straight outta 9wm

implementation

const
  AllButtonMask = ( Button1Mask or Button2Mask or Button3Mask or Button4Mask or Button5Mask );
  ButtonMask    = ( ButtonPressMask or ButtonReleaseMask );
  DragMask      = ( ButtonMask or ButtonMotionMask );
  MenuMask      = ( ButtonMask or ButtonMotionMask or ExposureMask );
  MenuGrabMask  = ( ButtonMask or ButtonMotionMask or StructureNotifyMask );

function errorHandler(d: PDisplay; e: PXErrorEvent): cint; cdecl;
var
  msg: array[0..99] of Char;
  number: array[0..29] of Char;
  request: array[0..99] of Char;
begin
    if m_initialising and (e^.request_code = X_ChangeWindowAttributes) and
	(e^.error_code = BadAccess) then
    begin
	WriteLn('fpwm error: Is another window manager running?');
	exit(1);
    end;

    // ugh
    if (ignoreBadWindowErrors = True) and (e^.error_code = BadWindow) then Exit(0);

    XGetErrorText(d, e^.error_code, msg, 100);
    WriteLn(Format('fpwm error: %d', [e^.request_code]));
    XGetErrorDatabaseText(d, 'XRequest', number, '', request, 100);

    if (request[0] = #0) then
      request := Format('<request-code-%x>', [e^.request_code]);
    WriteLn(Format('fpwm: %s (%d): %s', [request, e^.resourceid, msg]));

    if (m_initialising) then
    begin
	WriteLn('fpwm: failure during initialisation, abandoning');
	exit(1);
    end;

    Result := 0;
end;

function nobuttons(e: PXButtonEvent): Boolean; // straight outta 9wm
var
  state: cint;
begin
  state := (e^.state and AllButtonMask);
  Result := (e^._type = ButtonRelease) and (0 <> state and (state - 1));
  Result := not Result;
end;

{$I fpwmmanager_inline_manager.inc}
{$I fpwmmanager_inline_client.inc}
{$I fpwmmanager_inline_border.inc}

{$I fpwmmanager.inc}
{$I fpwmbuttons_manager.inc}
{$I fpwmevents_manager.inc}

{$I fpwmclient.inc}
{$I fpwmbuttons_client.inc}
{$I fpwmevents_client.inc}

{$I fpwmborder.inc}
{$I fpwmbuttons_border.inc}
{$I fpwmevents_border.inc}

end.
