{ Copyright 2010 - Felipe Monteiro de Carvalho
  Translation of the Xorg Xproto.h C header to Pascal }
{ Definitions for the X window system used by server }
{
 * This packet-construction scheme makes the following assumptions:
 *
 * 1. The compiler is able
 * to generate code which addresses one- and two-byte quantities.
 * In the worst case, this would be done with bit-fields.  If bit-fields
 * are used it may be necessary to reorder the request fields in this file,
 * depending on the order in which the machine assigns bit fields to
 * machine words.  There may also be a problem with sign extension,
 * as K+R specify that bitfields are always unsigned.
 *
 * 2. 2- and 4-byte fields in packet structures must be ordered by hand
 * such that they are naturally-aligned, so that no compiler will ever
 * insert padding bytes.
 *
 * 3. All packets are hand-padded to a multiple of 4 bytes, for
 * the same reason.
}
{***********************************************************

Copyright 1987, 1998  The Open Group

Permission to use, copy, modify, distribute, and sell this software and its
documentation for any purpose is hereby granted without fee, provided that
the above copyright notice appear in all copies and that both that
copyright notice and this permission notice appear in supporting
documentation.

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of The Open Group shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from The Open Group.


Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Digital not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************}
unit xproto;

{$mode delphi}

interface

uses x, xmd;

{#include <X11/Xmd.h>
#include <X11/Xprotostr.h>}

{
 * Define constants for the sizes of the network packets.  The sz_ prefix is
 * used instead of something more descriptive so that the symbols are no more
 * than 32 characters in length (which causes problems for some compilers).
 }
const
  sz_xSegment = 8;
  sz_xPoint = 4;
  sz_xRectangle = 8;
  sz_xArc = 12;
  sz_xConnClientPrefix = 12;
  sz_xConnSetupPrefix = 8;
  sz_xConnSetup = 32;
  sz_xPixmapFormat = 8;
  sz_xDepth = 8;
  sz_xVisualType = 24;
  sz_xWindowRoot = 40;
  sz_xTimecoord = 8;
  sz_xHostEntry = 4;
  sz_xCharInfo = 12;
  sz_xFontProp = 8;
  sz_xTextElt = 2;
  sz_xColorItem = 12;
  sz_xrgb = 8;
  sz_xGenericReply = 32;
  sz_xGetWindowAttributesReply = 44;
  sz_xGetGeometryReply = 32;
  sz_xQueryTreeReply = 32;
  sz_xInternAtomReply = 32;
  sz_xGetAtomNameReply = 32;
  sz_xGetPropertyReply = 32;
  sz_xListPropertiesReply = 32;
  sz_xGetSelectionOwnerReply = 32;
  sz_xGrabPointerReply = 32;
  sz_xQueryPointerReply = 32;
  sz_xGetMotionEventsReply = 32;
  sz_xTranslateCoordsReply = 32;
  sz_xGetInputFocusReply = 32;
  sz_xQueryKeymapReply = 40;
  sz_xQueryFontReply = 60;
  sz_xQueryTextExtentsReply = 32;
  sz_xListFontsReply = 32;
  sz_xGetFontPathReply = 32;
  sz_xGetImageReply = 32;
  sz_xListInstalledColormapsReply = 32;
  sz_xAllocColorReply = 32;
  sz_xAllocNamedColorReply = 32;
  sz_xAllocColorCellsReply = 32;
  sz_xAllocColorPlanesReply = 32;
  sz_xQueryColorsReply = 32;
  sz_xLookupColorReply = 32;
  sz_xQueryBestSizeReply = 32;
  sz_xQueryExtensionReply = 32;
  sz_xListExtensionsReply = 32;
  sz_xSetMappingReply = 32;
  sz_xGetKeyboardControlReply = 52;
  sz_xGetPointerControlReply = 32;
  sz_xGetScreenSaverReply = 32;
  sz_xListHostsReply = 32;
  sz_xSetModifierMappingReply = 32;
  sz_xError = 32;
  sz_xEvent = 32;
  sz_xKeymapEvent = 32;
  sz_xReq = 4;
  sz_xResourceReq = 8;
  sz_xCreateWindowReq = 32;
  sz_xChangeWindowAttributesReq = 12;
  sz_xChangeSaveSetReq = 8;
  sz_xReparentWindowReq = 16;
  sz_xConfigureWindowReq = 12;
  sz_xCirculateWindowReq = 8;
  sz_xInternAtomReq = 8;
  sz_xChangePropertyReq = 24;
  sz_xDeletePropertyReq = 12;
  sz_xGetPropertyReq = 24;
  sz_xSetSelectionOwnerReq = 16;
  sz_xConvertSelectionReq = 24;
  sz_xSendEventReq = 44;
  sz_xGrabPointerReq = 24;
  sz_xGrabButtonReq = 24;
  sz_xUngrabButtonReq = 12;
  sz_xChangeActivePointerGrabReq = 16;
  sz_xGrabKeyboardReq = 16;
  sz_xGrabKeyReq = 16;
  sz_xUngrabKeyReq = 12;
  sz_xAllowEventsReq = 8;
  sz_xGetMotionEventsReq = 16;
  sz_xTranslateCoordsReq = 16;
  sz_xWarpPointerReq = 24;
  sz_xSetInputFocusReq = 12;
  sz_xOpenFontReq = 12;
  sz_xQueryTextExtentsReq = 8;
  sz_xListFontsReq = 8;
  sz_xSetFontPathReq = 8;
  sz_xCreatePixmapReq = 16;
  sz_xCreateGCReq = 16;
  sz_xChangeGCReq = 12;
  sz_xCopyGCReq = 16;
  sz_xSetDashesReq = 12;
  sz_xSetClipRectanglesReq = 12;
  sz_xCopyAreaReq = 28;
  sz_xCopyPlaneReq = 32;
  sz_xPolyPointReq = 12;
  sz_xPolySegmentReq = 12;
  sz_xFillPolyReq = 16;
  sz_xPutImageReq = 24;
  sz_xGetImageReq = 20;
  sz_xPolyTextReq = 16;
  sz_xImageTextReq = 16;
  sz_xCreateColormapReq = 16;
  sz_xCopyColormapAndFreeReq = 12;
  sz_xAllocColorReq = 16;
  sz_xAllocNamedColorReq = 12;
  sz_xAllocColorCellsReq = 12;
  sz_xAllocColorPlanesReq = 16;
  sz_xFreeColorsReq = 12;
  sz_xStoreColorsReq = 8;
  sz_xStoreNamedColorReq = 16;
  sz_xQueryColorsReq = 8;
  sz_xLookupColorReq = 12;
  sz_xCreateCursorReq = 32;
  sz_xCreateGlyphCursorReq = 32;
  sz_xRecolorCursorReq = 20;
  sz_xQueryBestSizeReq = 12;
  sz_xQueryExtensionReq = 8;
  sz_xChangeKeyboardControlReq = 8;
  sz_xBellReq = 4;
  sz_xChangePointerControlReq = 12;
  sz_xSetScreenSaverReq = 12;
  sz_xChangeHostsReq = 8;
  sz_xListHostsReq = 4;
  sz_xChangeModeReq = 4;
  sz_xRotatePropertiesReq = 12;
  sz_xReply = 32;
  sz_xGrabKeyboardReply = 32;
  sz_xListFontsWithInfoReply = 60;
  sz_xSetPointerMappingReply = 32;
  sz_xGetKeyboardMappingReply = 32;
  sz_xGetPointerMappingReply = 32;
  sz_xGetModifierMappingReply = 32;
  sz_xListFontsWithInfoReq = 8;
  sz_xPolyLineReq = 12;
  sz_xPolyArcReq = 12;
  sz_xPolyRectangleReq = 12;
  sz_xPolyFillRectangleReq = 12;
  sz_xPolyFillArcReq = 12;
  sz_xPolyText8Req = 16;
  sz_xPolyText16Req = 16;
  sz_xImageText8Req = 16;
  sz_xImageText16Req = 16;
  sz_xSetPointerMappingReq = 4;
  sz_xForceScreenSaverReq = 4;
  sz_xSetCloseDownModeReq = 4;
  sz_xClearAreaReq = 16;
  sz_xSetAccessControlReq = 4;
  sz_xGetKeyboardMappingReq = 8;
  sz_xSetModifierMappingReq = 4;
  sz_xPropIconSize = 24;
  sz_xChangeKeyboardMappingReq = 8;


{ For the purpose of the structure definitions in this file,
we must redefine the following types in terms of Xmd.h's types, which may
include bit fields.  All of these are #undef'd at the end of this file,
restoring the definitions in X.h.  }

{  Window CARD32
  Drawable CARD32
  Font CARD32
  Pixmap CARD32
  Cursor CARD32
  Colormap CARD32
  GContext CARD32
  Atom CARD32
  VisualID CARD32
  Time CARD32
  KeyCode CARD8
  KeySym CARD32}

//  X_TCP_PORT 6000     /* add display number */

  xTrue        = 1;
  xFalse       = 0;


(*type
  //typedef CARD16 KeyButMask;

{*****************
   connection setup structure.  This is followed by
   numRoots xWindowRoot structs.
*****************}

xConnClientPrefix = record
   byteOrder: CARD8;
   pad: byte;
   majorVersion {B16}, minorVersion: CARD16; // B16
   nbytesAuthProto: CARD16;// B16;     /* Authorization protocol */
   nbytesAuthString:CARD16;// B16;     /* Authorization string */
   pad2: CARD16;// B16;
 end;

 xConnSetupPrefix = record
  success: CARD8; //* num bytes in string following if failure */
  legthReason: byte;
  majorVersion, minorVersion: CARD16;//B16;
  length: CARD16;//B16; /* 1/4 additional bytes in setup info */
 end;

 xConnSetup = record
  release: CARD32;//B32;
  ridBase, ridMask: CARD32;//B32;
  motionBufferSize: CARD32;//B32;
  nbytesVendor: CARD16;//B16;  /* number of bytes in vendor string */
  maxRequestSize: CARD16;//B16;
  numRoots: CARD8;     //* number of roots structs to follow */
  numFormats: CARD8;  //* number of pixmap formats */
  imageByteOrder: CARD8;  //* LSBFirst, MSBFirst */
  bitmapBitOrder: CARD8;  //* LeastSignificant, MostSign...*/
  bitmapScanlineUnit, bitmapScanlinePad: CARD8;      //* 8, 16, 32 */
  minKeyCode, maxKeyCode: TKeyCode;
  pad2: CARD32;//B32;
 end;

 xPixmapFormat = record
  depth: CARD8;
  bitsPerPixel: CARD8;
  scanLinePad: CARD8;
  pad1: CARD8;
  pad2: CARD32;//B32;
 end;

 //* window root */

 xDepth = record
  depth: CARD8;
  pad1: CARD8;
  nVisuals: CARD16;//B16;   /* number of xVisualType structures following */
  pad2: CARD32;//B32;
 end;

 xVisualType = record
   visualID: TVisualID;//B32;
   c_class: CARD8;
   bitsPerRGB: CARD8;
   colormapEntries: CARD16;//B16;
   redMask, greenMask, blueMask: CARD32;//B32;
   pad: CARD32;//B32;
 end;


xWindowRoot = record
  windowId: TWindow;//B32;
  defaultColormap: TColormap;//B32;
  whitePixel, blackPixel: CARD32;//B32;
  currentInputMask: CARD32;//B32;
  pixWidth, pixHeight: CARD16;//B16;
  mmWidth, mmHeight: CARD16;//B16;
  minInstalledMaps, maxInstalledMaps: CARD16;//B16;
  rootVisualID: TVisualID;//B32;
  backingStore: CARD8;
  saveUnders: TBOOL;
  rootDepth: CARD8;
  nDepths: CARD8; //* number of xDepth structures following */
end;




{*****************************************************************
 * Structure Defns
 *   Structures needed for replies
 *****************************************************************}

//* Used in GetMotionEvents */

xTimecoord = record
  time: CARD32;//B32;
  x, y: INT16;//B16;
end;

xHostEntry = record
  family: CARD8;
  pad: byte;
  length: CARD16;//B16;
end;

xCharInfo = record
  leftSideBearing, rightSideBearing, characterWidth,
	  ascent, descent: INT16;//B16;
  attributes: CARD16;//B16;
end;

xFontProp = record
  name: TAtom;//B32;
  value: CARD32;//B32;
end;

{/*
 * non-aligned big-endian font ID follows this struct
 */}

xTextElt = record    //* followed by string */
  len: CARD8;   {/* number of *characters* in string, or FontChange (255)
		   for font change, or 0 if just delta given */}
  delta: INT8;
end;

xColorItem = record
  pixel: CARD32;//B32;
  red, green, blue: CARD16;//B16;
  flags: CARD8;   //* DoRed, DoGreen, DoBlue booleans */
  pad: CARD8;
end;

xrgb = record
  red, green, blue, pad: CARD16;
end;

{*

typedef CARD8 KEYCODE;

 *}

{/*****************
 * XRep:
 *    meant to be 32 byte quantity
 *****************/

/* GenericReply is the common format of all replies.  The "data" items
   are specific to each individual reply type. */}

xGenericReply = record
  _type: byte;     //* X_Reply */
  data1: byte;  //* depends on reply type */
  sequenceNumber: CARD16;//B16; /* of last request received by server */
  length: CARD32;//B32;   /* 4 byte quantities beyond size of GenericReply */
  data00: CARD32;//B32;
  data01: CARD32;//B32;
  data02: CARD32;//B32;
  data03: CARD32;//B32;
  data04: CARD32;//B32;
  data05: CARD32;//B32;
end;


//* Individual reply formats. */

xGetWindowAttributesReply = record
  _type: byte; //* X_Reply */
  backingStore: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;   /* NOT 0; this is an extra-large reply */
  visualID: TVisualID;//B32;
  c_class: CARD16;//B16;
  bitGravity: CARD8;
  winGravity: CARD8;
  backingBitPlanes: CARD32;//B32;
  backingPixel: CARD32;//B32;
  saveUnder: TBOOL;
  mapInstalled: TBOOL;
  mapState: CARD8;
  override: TBOOL;
  colormap: TColormap;//B32;
  allEventMasks: CARD32;//B32;
  yourEventMask: CARD32;//B32;
  doNotPropagateMask: CARD16;//B16;
  pad: CARD16;//B16;
end;

xGetGeometryReply = record
  _type: BYTE;   //* X_Reply */
  depth: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  root: TWindow;//B32;
  x, y: INT16;//B16;
  width, height: CARD16;//B16;
  borderWidth: CARD16;//B16;
  pad1: CARD16;//B16;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
end;

xQueryTreeReply = record
  _type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  root, parent: TWindow;//B32;
  nChildren: CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
end;

xInternAtomReply = record
  _type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32; /* 0 */
  atom: TAtom;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
end;

xGetAtomNameReply = record
  _type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* of additional bytes */
  nameLength: CARD16;//B16;  /* # of characters in name */
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xGetPropertyReply = record
  _type: BYTE;  //* X_Reply */
  format: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32; /* of additional bytes */
  propertyType: TAtom;//B32;
  bytesAfter: CARD32;//B32;
  nItems: CARD32;//B32; /* # of 8, 16, or 32-bit entities in reply */
  pad1: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
end;

xListPropertiesReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nProperties: CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xGetSelectionOwnerReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  owner: Window;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
end;

xGrabPointerReply = record
  type: BYTE;  //* X_Reply */
  status: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  pad1: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
end;



    {
typedef xGrabPointerReply xGrabKeyboardReply;
}

xQueryPointerReply = record
  type: BYTE;  //* X_Reply */
  sameScreen: BOOL;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  root, child: Window;//B32
  rootX, rootY, winX, winY: INT16;//B16;
  mask: CARD16;//B16;
  pad1: CARD16;//B16;
  pad: CARD32;//B32;
end;

xGetMotionEventsReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nEvents: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
end;

xTranslateCoordsReply = record
  type: BYTE;  //* X_Reply */
  sameScreen: BOOL;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32; /* 0 */
  child: Window;//B32;
  dstX, dstY: INT16;//B16;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
end;

xGetInputFocusReply = record
  type: BYTE;  //* X_Reply */
  revertTo: CARD8;
  sequenceNumber: CARD16//B16;
  length: CARD32;//B32;  /* 0 */
  focus: Window ;//B32;
  pad1: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
end;

xQueryKeymapReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 2, NOT 0; this is an extra-large reply */
  map[32]: BYTE;
end;


//* Warning: this MUST match (up to component renaming) xListFontsWithInfoReply */

xQueryFontReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  //* definitely > 0, even if "nCharInfos" is 0 */
  minBounds: xCharInfo;
  {$ifndef WORD64}
    walign1: CARD32;//B32;
  {$endif}
    maxBounds: xCharInfo;
  {$ifndef WORD64}
    walign2: CARD32;//B32;
  {$endif}
    minCharOrByte2, maxCharOrByte2: CARD16;//B16;
    defaultChar: CARD16;//B16;
    nFontProps: CARD16;//B16;  //* followed by this many xFontProp structures */
    drawDirection: CARD8;
    minByte1, maxByte1: CARD8;
    allCharsExist: BOOL;
    fontAscent, fontDescent: INT16;//B16;
    nCharInfos: CARD32;//B32; /* followed by this many xCharInfo structures */
end;


xQueryTextExtentsReply = record
  type: BYTE;  //* X_Reply */
  drawDirection: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  fontAscent, fontDescent: INT16;//B16;
  overallAscent, overallDescent: INT16;//B16;
  overallWidth, overallLeft, overallRight: INT32;//B32;
  pad: CARD32;//B32;
end;

xListFontsReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nFonts: CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;


//* Warning: this MUST match (up to component renaming) xQueryFontReply */

xListFontsWithInfoReply = record
  type: BYTE;  //* X_Reply */
  nameLength: CARD8;  //* 0 indicates end-of-reply-sequence */
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* definitely > 0, even if "nameLength" is 0 */
  minBounds: xCharInfo;
{$ifndef WORD64}
  walign1: CARD32;//B32;
{$endif}
  maxBounds: xCharInfo;
{$ifndef WORD64}
  walign2; CARD32;//B32;
{$endif}
  minCharOrByte2, maxCharOrByte2: CARD16;//B16;
  defaultChar: CARD16;//B16;
  nFontProps: CARD16;//B16;  /* followed by this many xFontProp structures */
  drawDirection: CARD8;
  minByte1, maxByte1: CARD8;
  allCharsExist: BOOL;
  fontAscent, fontDescent: INT16;//B16;
  nReplies: CARD32;//B32;   /* hint as to how many more replies might be coming */
end;

xGetFontPathReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nPaths: CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xGetImageReply = record
  type: BYTE;  //* X_Reply */
  depth: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  visual: VisualID;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xListInstalledColormapsReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nColormaps; CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xAllocColorReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;   /* 0 */
  red, green, blue: CARD16;//B16;
  pad2: CARD16;//B16;
  pixel: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
end;

xAllocNamedColorReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  pixel: CARD32;//B32;
  exactRed, exactGreen, exactBlue: CARD16;//B16;
  screenRed, screenGreen, screenBlue: CARD16;//B16;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
end;

xAllocColorCellsReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nPixels, nMasks: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xAllocColorPlanesReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nPixels: CARD16;//B16;
  pad2: CARD16;//B16;
  redMask, greenMask, blueMask: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
end;

xQueryColorsReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  nColors: CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xLookupColorReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  exactRed, exactGreen, exactBlue: CARD16;//B16;
  screenRed, screenGreen, screenBlue: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
end;

xQueryBestSizeReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;  /* 0 */
  width, height: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xQueryExtensionReply = record
  type: BYTE;  //* X_Reply */
  pad1: BYTE;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32; /* 0 */
  present: BOOL;
  major_opcode: CARD8;
  first_event: CARD8;
  first_error: CARD8;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xListExtensionsReply = record
  type: BYTE;  //* X_Reply */
  nExtensions: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xSetMappingReply = record
  type: BYTE;  //* X_Reply */
  success: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

{typedef xSetMappingReply xSetPointerMappingReply;
typedef xSetMappingReply xSetModifierMappingReply;}

xGetPointerMappingReply = record
  type: BYTE;  //* X_Reply */
  nElts: CARD8;  //* how many elements does the map have */
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xGetKeyboardMappingReply = record
  type: BYTE;
  keySymsPerKeyCode: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

xGetModifierMappingReply
  type: BYTE;
  numKeyPerModifier: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  pad1: CARD32;//B32;
  pad2: CARD32;//B32;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
end;

xGetKeyboardControlReply = record
    type: BYTE;  //* X_Reply */
    globalAutoRepeat: BOOL;
    sequenceNumber: CARD16;//B16;
    length: CARD32;//B32;  /* 5 */
    ledMask: CARD32;//B32;
    keyClickPercent, bellPercent: CARD8;
    bellPitch, bellDuration: CARD16;//B16;
    pad: CARD16;//B16;
    map[32]: BYTE;  //* bit masks start here */
end;/

xGetPointerControlReply = record
    type: BYTE;  //* X_Reply */
    pad1: BYTE;
    sequenceNumber: CARD16;//B16;
    length: CARD32;//B32;  /* 0 */
    accelNumerator, accelDenominator: CARD16;//B16;
    threshold: CARD16;//B16;
    pad2: CARD16;//B16;
    pad3: CARD32;//B32;
    pad4: CARD32;//B32;
    pad5: CARD32;//B32;
    pad6: CARD32;//B32;
end;

xGetScreenSaverReply = record
    type: BYTE;  //* X_Reply */
    pad1: BYTE;
    sequenceNumber: CARD16;//B16;
    length: CARD32;//B32;  /* 0 */
    timeout, interval: CARD16;//B16;
    preferBlanking: BOOL;
    allowExposures: BOOL;
    pad2: CARD16;//B16;
    pad3: CARD32;//B32;
    pad4: CARD32;//B32;
    pad5: CARD32;//B32;
    pad6: CARD32;//B32;
end;

xListHostsReply
    type: BYTE;  //* X_Reply */
    enabled: BOOL;
    sequenceNumber: CARD16;//B16;
    length: CARD32;//B32;
    nHosts: CARD16;//B16;
    pad1: CARD16;//B16;
    pad3: CARD32;//B32;
    pad4: CARD32;//B32;
    pad5: CARD32;//B32;
    pad6: CARD32;//B32;
    pad7: CARD32;//B32;
end;




{/*****************************************************************
 * Xerror
 *    All errors  are 32 bytes
 *****************************************************************/}

xError = record
    type: BYTE;                  //* X_Error */
    errorCode: BYTE;
    sequenceNumber: CARD16;//B16;       /* the nth request from this client */
    resourceID: CARD32;//B32;
    minorCode: CARD16;//B16;
    majorCode: CARD8;
    pad1: BYTE;
    pad3: CARD32;//B32;
    pad4: CARD32;//B32;
    pad5: CARD32;//B32;
    pad6: CARD32;//B32;
    pad7: CARD32;//B32;
end;

{/*****************************************************************
 * xEvent
 *    All events are 32 bytes
 *****************************************************************/}
(*
typedef struct _xEvent {
    union {
	struct {
	    type: BYTE;
	    detail: BYTE;
	    sequenceNumber: CARD16;//B16;
	    } u;
	struct {
            pad00: CARD32;//B32;
	    time: Time;//B32;
	    root, event, child: Window;//B32;
	    rootX, rootY, eventX, eventY: INT16;//B16;
	    state: KeyButMask;//B16;
	    sameScreen: BOOL;
	    pad1: BYTE;
	} keyButtonPointer;
	struct {
            pad00: CARD32;//B32;
            time: Time;//B32;
	    root, event, child: Window;//B32;
	    rootX, rootY, eventX, eventY: INT16;//B16;
	    state: KeyButMask;//B16;
	    mode: BYTE; 			/* really XMode */
	    flags: BYTE;		/* sameScreen and focus booleans, packed together */
  ELFlagFocus        (1<<0)
  ELFlagSameScreen   (1<<1)
	} enterLeave;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    mode: BYTE; 			/* really XMode */
	    pad1, pad2, pad3: BYTE;
	} focus;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    x, y, width, height: CARD16;//B16;
	    count: CARD16;//B16;
	    pad2: CARD16;//B16;
	} expose;
	struct {
            pad00: CARD32;//B32;
	    drawable: Drawable;//B32;
	    x, y, width, height: CARD16;//B16;
	    minorEvent: CARD16;//B16;
	    count: CARD16;//B16;
	    majorEvent: BYTE;
	    pad1, pad2, pad3: BYTE;
	} graphicsExposure;
	struct {
            pad00: CARD32;//B32;
	    drawable: Drawable;//B32;
	    minorEvent: CARD16;//B16;
	    majorEvent: BYTE;
	    bpad: BYTE;
	} noExposure;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    state: CARD8;
	    pad1, pad2, pad3: BYTE;
	} visibility;
	struct {
            pad00: CARD32;//B32;
	    parent, window: Window;//B32;
	    x, y: INT16;//B16;
	    width, height, borderWidth: CARD16;//B16;
	    override: BOOL;
	    bpad: BYTE;
        } createNotify;
/*
 * The event fields in the structures for DestroyNotify, UnmapNotify,
 * MapNotify, ReparentNotify, ConfigureNotify, CirculateNotify, GravityNotify,
 * must be at the same offset because server internal code is depending upon
 * this to patch up the events before they are delivered.
 * Also note that MapRequest, ConfigureRequest and CirculateRequest have
 * the same offset for the event window.
 */
	struct {
            pad00: CARD32;//B32;
	    event, window: Window;//B32;
	} destroyNotify;
	struct {
            pad00: CARD32;//B32;
	    event, window: Window;//B32;
	    fromConfigure: BOOL;
	    pad1, pad2, pad3: BYTE;
        } unmapNotify;
	struct {
            pad00: CARD32;//B32;
	    event, window: Window;//B32;
	    override: BOOL;
	    pad1, pad2, pad3: BYTE;
        } mapNotify;
	struct {
            pad00: CARD32;//B32;
	    parent, window: Window;//B32;
        } mapRequest;
	struct {
            pad00: CARD32;//B32;
	    event, window, parent: Window;//B32;
	    x, y: INT16;//B16;
	    override: BOOL;
	    pad1, pad2, pad3: BYTE;
	} reparent;
	struct {
            pad00: CARD32;//B32;
	    event, window, aboveSibling: Window;//B32;
	    x, y: INT16;//B16;
	    width, height, borderWidth: CARD16;//B16;
	    override: BOOL;
	    bpad: BYTE;
	} configureNotify;
	struct {
            pad00: CARD32;//B32;
	    parent, window, sibling: Window;//B32;
	    x, y: INT16;//B16;
	    width, height, borderWidth: CARD16;//B16;
	    valueMask: CARD16;//B16;
	    pad1: CARD32;//B32;
	} configureRequest;
	struct {
            pad00: CARD32;//B32;
	    event, window: Window;//B32;
	    x, y: INT16;//B16;
	    pad1, pad2, pad3, pad4: CARD32;//B32;
	} gravity;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    width, height: CARD16;//B16;
	} resizeRequest;
	struct {
/* The event field in the circulate record is really the parent when this
   is used as a CirculateRequest instead of a CirculateNotify */
            pad00: CARD32;//B32;
	    event, window, parent: Window;//B32;
	    place: BYTE;			/* Top or Bottom */
	    pad1, pad2, pad3: BYTE;
	} circulate;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    atom: Atom;//B32;
	    time: Time;//B32;
	    state: BYTE;			/* NewValue or Deleted */
	    pad1: BYTE;
	    pad2: CARD16;//B16;
	} property;
	struct {
            pad00: CARD32;//B32;
            time: Time;//B32;
	    window: Window;//B32;
	    atom: Atom;//B32;
	} selectionClear;
	struct {
            pad00: CARD32;//B32;
            time: Time;//B32;
	    owner, requestor: Window;//B32;
	    selection, target, property: Atom;//B32;
	} selectionRequest;
	struct {
            pad00: CARD32;//B32;
            time: Time;//B32;
	    requestor: Window;//B32;
	    selection, target, property: Atom;//B32;
	} selectionNotify;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    colormap: Colormap;//B32;
#if defined(__cplusplus) || defined(c_plusplus)
	    c_new: BOOL;
#else
	    new: BOOL;
#endif
	    state: BYTE;			/* Installed or UnInstalled */
	    pad1, pad2: BYTE;
	} colormap;
	struct {
	    pad00: CARD32;//B32;
	    request: CARD8;
	    firstKeyCode: KeyCode;
	    count: CARD8;
	    pad1: BYTE;
	} mappingNotify;
	struct {
            pad00: CARD32;//B32;
	    window: Window;//B32;
	    union {
		struct {
		    type: Atom;//B32;
		    longs0: INT32;//B32;
		    longs1: INT32;//B32;
                    longs2: INT32;//B32;
                    longs3: INT32;//B32;
                    longs4: INT32;//B32;
		} l;
		struct {
		    type: Atom;//B32;
		    shorts0: INT16;//B16;
                    shorts1: INT16;//B16;
                    shorts2: INT16;//B16;
                    shorts3: INT16;//B16;
                    shorts4: INT16;//B16;
                    shorts5: INT16;//B16;
                    shorts6: INT16;//B16;
                    shorts7: INT16;//B16;
                    shorts8: INT16;//B16;
                    shorts9: INT16;//B16;
		} s;
		struct {
		    type: Atom;//B32;
		    bytes[20]: INT8;
		} b;
	    } u;
	} clientMessage;
    } u;
} xEvent;

/*********************************************************
 *
 * Generic event
 *
 * Those events are not part of the core protocol spec and can be used by
 * various extensions.
 * type is always GenericEvent
 * extension is the minor opcode of the extension the event belongs to.
 * evtype is the actual event type, unique __per extension__.
 *
 * GenericEvents can be longer than 32 bytes, with the length field
 * specifying the number of 4 byte blocks after the first 32 bytes.
 *
 *
 */
*}


xGenericEvent = record
  type: BYTE;
  extension: CARD8;
  sequenceNumber: CARD16;//B16;
  length: CARD32;//B32;
  evtype: CARD16;//B16;
  pad2: CARD16;//B16;
  pad3: CARD32;//B32;
  pad4: CARD32;//B32;
  pad5: CARD32;//B32;
  pad6: CARD32;//B32;
  pad7: CARD32;//B32;
end;

{/* KeymapNotify events are not included in the above union because they
   are different from all other events: they do not have a "detail"
   or "sequenceNumber", so there is room for a 248-bit key mask. */}

xKeymapEvent = record
    type: BYTE;
    map[31]: BYTE;
end;

  {XEventSize (sizeof(xEvent))}

{/* XReply is the union of all the replies above whose "fixed part"
fits in 32 bytes.  It does NOT include GetWindowAttributesReply,
QueryFontReply, QueryKeymapReply, or GetKeyboardControlReply
ListFontsWithInfoReply */}

xReply = record
    generic: xGenericReply;
    geom: xGetGeometryReply;
    tree: xQueryTreeReply;
    atom: xInternAtomReply;
    atomName: xGetAtomNameReply;
    property: xGetPropertyReply;
    listProperties: xListPropertiesReply;
    selection: xGetSelectionOwnerReply;
    grabPointer: xGrabPointerReply;
    grabKeyboard: xGrabKeyboardReply;
    pointer: xQueryPointerReply;
    motionEvents: xGetMotionEventsReply;
    coords: xTranslateCoordsReply;
    inputFocus: xGetInputFocusReply;
    textExtents: xQueryTextExtentsReply;
    fonts: xListFontsReply;
    fontPath: xGetFontPathReply;
    image: xGetImageReply;
    colormaps: xListInstalledColormapsReply;
    allocColor: xAllocColorReply;
    allocNamedColor: xAllocNamedColorReply;
    colorCells: xAllocColorCellsReply;
    colorPlanes: xAllocColorPlanesReply;
    colors: xQueryColorsReply;
    lookupColor: xLookupColorReply;
    bestSize: xQueryBestSizeReply;
    extension: xQueryExtensionReply;
    extensions: xListExtensionsReply;
    setModifierMapping: xSetModifierMappingReply;
    getModifierMapping: xGetModifierMappingReply;
    setPointerMapping: xSetPointerMappingReply;
    getKeyboardMapping: xGetKeyboardMappingReply;
    getPointerMapping: xGetPointerMappingReply;
    pointerControl: xGetPointerControlReply;
    screenSaver: xGetScreenSaverReply;
    hosts: xListHostsReply;
    error: xError;
    event: xEvent;
end;




{/*****************************************************************
 * REQUESTS
 *****************************************************************/}


//* Request structure */

xReq = record
	reqType: CARD8;
	data: CARD8;            //* meaning depends on request type */
	length: CARD16;//B16;        /* length in 4 bytes quantities
	                        // of whole request, including this header */
end;


{/*****************************************************************
 *  structures that follow request.
 *****************************************************************/}

{/* ResourceReq is used for any request which has a resource ID
   (or Atom or Time) as its one and only argument.  */}

xResourceReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    id: CARD32;//B32;  /* a Window, Drawable, Font, GContext, Pixmap, etc. */

xCreateWindowReq = record
    reqType: CARD8;
    depth: CARD8;
    length: CARD16;//B16;
    wid, parent: Window;//B32;
    x, y: INT16;//B16;
    width, height, borderWidth: CARD16;//B16;
{#if defined(__cplusplus) || defined(c_plusplus)}
    c_class: CARD16;//B16;
{$else}
    class: CARD16;//B16;
{$endif}
    visual: VisualID;//B32;
    mask: CARD32;//B32;
end;

xChangeWindowAttributesReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    window: Window;//B32;
    valueMask: CARD32;// B32;
end;

xChangeSaveSetReq = record
    reqType: CARD8;
    mode: BYTE;
    length: CARD16;//B16;
    window: Window;//B32;
end;

xReparentWindowReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    window, parent: Window;//B32;
    x, y: INT16;//B16;
end;

xConfigureWindowReq = record
    reqType: CARD8;
    pad: CARD8;
    length: CARD16;//B16;
    window: Window;//B32;
    mask: CARD16;//B16;
    pad2: CARD16;//B16;
end;

xCirculateWindowReq = record
    reqType: CARD8;
    direction: CARD8;
    length: CARD16;//B16;
    window: Window;//B32;
end;

xInternAtomReq = record  //* followed by padded string */
    reqType: CARD8;
    onlyIfExists: BOOL;
    length: CARD16;//B16;
    nbytes: CARD16;//B16;    /* number of bytes in string */
    pad: CARD16;//B16;
end;

xChangePropertyReq = record
    reqType: CARD8;
    mode: CARD8;
    length: CARD16;//B16;
    window: Window;//B32;
    property, type: Atom;//B32;
    format: CARD8;
    pad[3]: BYTE;
    nUnits: CARD32;//B32;   /* length of stuff following, depends on format */
end;

xDeletePropertyReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    window: Window;//B32;
    property: Atom;//B32;
end;

xGetPropertyReq = record
    reqType: CARD8;
{if defined(__cplusplus) || defined(c_plusplus)}
    c_delete: BOOL;
{$else}
    delete: BOOL;
{$endif}
    length: CARD16;//B16;
    window: Window;//B32;
    property, type: Atom;//B32;
    longOffset: CARD32;//B32;
    longLength: CARD32;//B32;
end;

xSetSelectionOwnerReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    window: Window;//B32;
    selection: Atom;//B32;
    time: Time;//B32;
end;

xConvertSelectionReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    requestor: Window;//B32;
    selection, target, property: Atom;//B32;
    time: Time;//B32;
end;

xSendEventReq = record
    reqType: CARD8;
    BOOL propagate;
    length: CARD16;//B16;
    Window destination B32;
    CARD32 eventMask B32;
{$ifdef WORD64}
    //* the structure should have been quad-aligned */
    eventdata[SIZEOF(xEvent)]: BYTE;
{$else}
    event: xEvent;
{$endif} //* WORD64 */
end;

xGrabPointerReq = record
    reqType: CARD8;
    BOOL ownerEvents;
    length: CARD16;//B16;
    grabWindow: Window;//B32;
    eventMask: CARD16;//B16;
    pointerMode, keyboardMode: BYTE;
    confineTo: Window;//B32;
    cursor: Cursor;//B32;
    time: Time;//B32;
end;

xGrabButtonReq = record
    reqType: CARD8;
    ownerEvents: BOOL;
    length: CARD16;//B16;
    grabWindow: Window;//B32;
    eventMask: CARD16;//B16;
    pointerMode, keyboardMode: BYTE;
    confineTo: Window;//B32;
    cursor: Cursor;//B32;
    button: CARD8;
    pad: BYTE;
    modifiers: CARD16;//B16;
end;

xUngrabButtonReq = record
    reqType: CARD8;
    button: CARD8;
    length: CARD16;//B16;
    grabWindow: Window;//B32;
    modifiers: CARD16;//B16;
    pad: CARD16;//B16;
end;

xChangeActivePointerGrabReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cursor: Cursor;//B32;
    time: Time;//B32;
    eventMask: CARD16;//B16;
    pad2: CARD16;//B16;
end;

xGrabKeyboardReq = record
    reqType: CARD8;
    ownerEvents: BOOL;
    length: CARD16;//B16;
    grabWindow: Window;//B32;
    time: Time;//B32;
    pointerMode, keyboardMode: BYTE;
    pad: CARD16;//B16;
end;

xGrabKeyReq = record
    reqType: CARD8;
    ownerEvents: BOOL;
    length: CARD16;//B16;
    grabWindow: Window;//B32;
    modifiers: CARD16;//B16;
    key: CARD8;
    pointerMode, keyboardMode: BYTE;
    pad1, pad2, pad3: BYTE;
end;

xUngrabKeyReq = record
    reqType: CARD8;
    key: CARD8;
    length: CARD16;//B16;
    grabWindow: Window;//B32;
    modifiers: CARD16;//B16;
    pad: CARD16;//B16;
end;

xAllowEventsReq = record
    reqType: CARD8;
    mode: CARD8;
    length: CARD16;//B16;
    time: Time;//B32;
end;

xGetMotionEventsReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    window: Window;//B32;
    start, stop: Time;//B32;
end;

xTranslateCoordsReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    srcWid, dstWid: Window;//B32;
    srcX, srcY: INT16;//B16;
end;

xWarpPointerReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    srcWid, dstWid: Window;//B32;
    srcX, srcY: INT16;//B16;
    srcWidth, srcHeight: CARD16;//B16;
    dstX, dstY: INT16;//B16;
end;

xSetInputFocusReq = record
    reqType: CARD8;
    revertTo: CARD8;
    length: CARD16;//B16;
    focus: Window;//B32;
    time: Time;//B32;
end;

xOpenFontReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    fid: Font;//B32;
    nbytes: CARD16;//B16;
    pad1, pad2: BYTE;	//* string follows on word boundary */
end;

xQueryTextExtentsReq = record
    reqType: CARD8;
    oddLength: BOOL;
    length: CARD16;//B16;
    fid: Font;//B32;
end;

xListFontsReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    maxNames: CARD16;//B16;
    nbytes: CARD16;//B16;  /* followed immediately by string bytes */
end;

//typedef xListFontsReq xListFontsWithInfoReq;

xSetFontPathReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    nFonts: CARD16;//B16;
    pad1, pad2: BYTE;	//* LISTofSTRING8 follows on word boundary */
end;

xCreatePixmapReq = record
    reqType: CARD8;
    depth: CARD8;
    length: CARD16;//B16;
    pid: Pixmap;//B32;
    drawable: Drawable;//B32;
    width, height: CARD16;//B16;
end;

xCreateGCReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    gc: GContext;//B32;
    drawable: Drawable;//B32;
    mask: CARD32;//B32;
end;

xChangeGCReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    gc: GContext;//B32;
    mask: CARD32;//B32;
end;

xCopyGCReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    srcGC, dstGC: GContext;//B32;
    mask: CARD32;//B32;
end;

xSetDashesReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    gc: GContext;//B32;
    dashOffset: CARD16;//B16;
    nDashes: CARD16;//B16;        /* length LISTofCARD8 of values following */
end;

xSetClipRectanglesReq = record
    reqType: CARD8;
    ordering: BYTE;
    length: CARD16;//B16;
    gc: GContext;//B32;
    xOrigin, yOrigin: INT16;//B16;
end;

xClearAreaReq = record
    reqType: CARD8;
    exposures: BOOL;
    length: CARD16;//B16;
    window: Window;//B32;
    x, y: INT16;//B16;
    width, height: CARD16;//B16;
end;

xCopyAreaReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    srcDrawable, dstDrawable: Drawable;//B32;
    gc: GContext;//B32;
    srcX, srcY, dstX, dstY: INT16;//B16;
    width, height: CARD16;//B16;
end;

xCopyPlaneReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    srcDrawable, dstDrawable: Drawable;//B32;
    gc: GContext;//B32;
    srcX, srcY, dstX, dstY: INT16;//B16;
    width, height: CARD16;//B16;
    bitPlane: CARD32;//B32;
end;

xPolyPointReq = record
    reqType: CARD8;
    coordMode: BYTE;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    gc: GContext;//B32;
end;

{typedef xPolyPointReq xPolyLineReq;  /* same request structure */}

//* The following used for PolySegment, PolyRectangle, PolyArc, PolyFillRectangle, PolyFillArc */

xPolySegmentReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    gc: GContext;//B32;
end;

{
typedef xPolySegmentReq xPolyArcReq;
typedef xPolySegmentReq xPolyRectangleReq;
typedef xPolySegmentReq xPolyFillRectangleReq;
typedef xPolySegmentReq xPolyFillArcReq;}

xFillPolyReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    gc: GContext;//B32;
    shape: BYTE;
    coordMode: BYTE;
    pad1: CARD;//B16;
end;

xPutImageReq = record
    reqType: CARD8;
    format: CARD8;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    gc: GContext;//B32;
    width, height: CARD16;//B16;
    dstX, dstY: INT16;//B16;
    leftPad: CARD8;
    depth: CARD8;
    pad: CARD16;//B16;
end;

xGetImageReq = record
    reqType: CARD8;
    format: CARD8;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    x, y: INT16;//B16;
    width, height: CARD16;//B16;
    planeMask: CARD32;//B32;
end;

//* the following used by PolyText8 and PolyText16 */

xPolyTextReq = record
    reqType: CARD8;
    pad: CARD8;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    gc: GContext;//B32;
    x, y: INT16;//B16;		/* items (xTextElt) start after struct */
end;

{typedef xPolyTextReq xPolyText8Req;
typedef xPolyTextReq xPolyText16Req;}

xImageTextReq = record
    reqType: CARD8;
    nChars: BYTE;
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    gc: GContext;//B32;
    x, y: INT16;//B16;
end;

{typedef xImageTextReq xImageText8Req;
typedef xImageTextReq xImageText16Req;}

xCreateColormapReq = record
    reqType: CARD8;
    alloc: BYTE;
    length: CARD16;//B16;
    mid: Colormap;//B32;
    window: Window;//B32;
    visual: VisualID;//B32;
end;

xCopyColormapAndFreeReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    mid: Colormap;//B32;
    srcCmap: Colormap;//B32;
end;

xAllocColorReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    red, green, blue: CARD16;//B16;
    pad2: CARD16;//B16;
end;

xAllocNamedColorReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    nbytes: CARD16;//B16;  /* followed by structure */
    pad1, pad2: BYTE;
end;

xAllocColorCellsReq = record
    reqType: CARD8;
    contiguous: BOOL;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    colors, planes: CARD16;//B16;
end;

xAllocColorPlanesReq = record
    reqType: CARD8;
    contiguous: BOOL;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    colors, red, green, blue: CARD16;//B16;
end;

xFreeColorsReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    planeMask: CARD32;//B32;
end;

xStoreColorsReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
end;

xStoreNamedColorReq = record
    reqType: CARD8;
    CARD8 flags;   /* DoRed, DoGreen, DoBlue, as in xColorItem */
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    pixel: CARD32;//B32;
    nbytes: CARD16;//B16;  /* number of name string bytes following structure */
    pad1, pad2: BYTE;
end;

xQueryColorsReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
end;

xLookupColorReq = record  //* followed  by string of length len */
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cmap: Colormap;//B32;
    nbytes: CARD16;//B16;  /* number of string bytes following structure*/
    pad1, pad2: BYTE;
end;

xCreateCursorReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cid: Cursor;//B32;
    source, mask: Pixmap;//B32;
    foreRed, foreGreen, foreBlue: CARD16;//B16;
    CARD16 backRed B16, backGreen B16, backBlue B16;
    x, y: CARD16;//B16;
end;

xCreateGlyphCursorReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cid: Cursor;//B32;
    source, mask: Font;//B32;
    sourceChar, maskChar: CARD16;//B16;
    foreRed, foreGreen, foreBlue: CARD16;//B16;
    backRed, backGreen, backBlue: CARD16;//B16;
end;

xRecolorCursorReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    cursor: Cursor;//B32;
    foreRed, foreGreen, foreBlue: CARD16;//B16;
    backRed, backGreen, backBlue: CARD16;//B16;
end;

xQueryBestSizeReq = record
    reqType: CARD8;
{#if defined(__cplusplus) || defined(c_plusplus)}
    c_class: CARD8;
{$else}
    class: CARD8;
{$endif}
    length: CARD16;//B16;
    drawable: Drawable;//B32;
    width, height: CARD16;//B16;
end;

xQueryExtensionReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    nbytes: CARD16;//B16;  /* number of string bytes following structure */
    pad1, pad2: BYTE;
end;

xSetModifierMappingReq = record
    reqType: CARD8;
    numKeyPerModifier: CARD8;
    length: CARD16;//B16;
end;

xSetPointerMappingReq = recod
    reqType: CARD8;
    nElts: CARD8;  //* how many elements in the map */
    length: CARD16;//B16;
end;

xGetKeyboardMappingReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    firstKeyCode: KeyCode;
    count: CARD8;
    pad1: CARD;//B16;
end;

xChangeKeyboardMappingReq = record
    reqType: CARD8;
    keyCodes: CARD8;
    length: CARD16;//B16;
    firstKeyCode: KeyCode;
    keySymsPerKeyCode: CARD8;
    pad1: CARD;//B16;
end;

xChangeKeyboardControlReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    mask: CARD32;//B32;
end;

xBellReq = record
    reqType: CARD8;
    percent: INT8;  //* -100 to 100 */
    length: CARD16;//B16;
end;

xChangePointerControlReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    accelNum, accelDenum: INT16;//B16;
    threshold: INT16;//B16;
    doAccel, doThresh: BOOL;
end;

xSetScreenSaverReq = record
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    timeout, interval: INT16;//B16;
    preferBlank, allowExpose: BYTE;
    pad2: CARD16;//B16;
end;

xChangeHostsReq = record
    reqType: CARD8;
    mode: BYTE;
    length: CARD16;//B16;
    hostFamily: CARD8;
    pad: BYTE;
    hostLength: CARD16;//B16;
end;

xListHostsReq = recod
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
end;

xChangeModeReq = record
    reqType: CARD8;
    mode: BYTE;
    length: CARD16;//B16;
end;

{typedef xChangeModeReq xSetAccessControlReq;
typedef xChangeModeReq xSetCloseDownModeReq;
typedef xChangeModeReq xForceScreenSaverReq;}

xRotatePropertiesReq = record /* followed by LIST of ATOM */
    reqType: CARD8;
    pad: BYTE;
    length: CARD16;//B16;
    window: Window;//B32;
    nAtoms: CARD16;//B16;
    nPositions: INT16;//B16;
end;
*)

{ Reply codes }
const
  X_Reply		= 1;		{ Normal reply }
  X_Error		= 0;		{ Error }

{ Request codes }

  X_CreateWindow               =   1;
  X_ChangeWindowAttributes     =   2;
  X_GetWindowAttributes        =   3;
  X_DestroyWindow              =   4;
  X_DestroySubwindows          =   5;
  X_ChangeSaveSet              =   6;
  X_ReparentWindow             =   7;
  X_MapWindow                  =   8;
  X_MapSubwindows              =   9;
  X_UnmapWindow                =  10;
  X_UnmapSubwindows            =  11;
  X_ConfigureWindow            =  12;
  X_CirculateWindow            =  13;
  X_GetGeometry                =  14;
  X_QueryTree                  =  15;
  X_InternAtom                 =  16;
  X_GetAtomName                =  17;
  X_ChangeProperty             =  18;
  X_DeleteProperty             =  19;
  X_GetProperty                =  20;
  X_ListProperties             =  21;
  X_SetSelectionOwner          =  22;
  X_GetSelectionOwner          =  23;
  X_ConvertSelection           =  24;
  X_SendEvent                  =  25;
  X_GrabPointer                =  26;
  X_UngrabPointer              =  27;
  X_GrabButton                 =  28;
  X_UngrabButton               =  29;
  X_ChangeActivePointerGrab    =  30;
  X_GrabKeyboard               =  31;
  X_UngrabKeyboard             =  32;
  X_GrabKey                    =  33;
  X_UngrabKey                  =  34;
  X_AllowEvents                =  35;
  X_GrabServer                 =  36;
  X_UngrabServer               =  37;
  X_QueryPointer               =  38;
  X_GetMotionEvents            =  39;
  X_TranslateCoords            =  40;
  X_WarpPointer                =  41;
  X_SetInputFocus              =  42;
  X_GetInputFocus              =  43;
  X_QueryKeymap                =  44;
  X_OpenFont                   =  45;
  X_CloseFont                  =  46;
  X_QueryFont                  =  47;
  X_QueryTextExtents           =  48;
  X_ListFonts                  =  49;
  X_ListFontsWithInfo          =  50;
  X_SetFontPath                =  51;
  X_GetFontPath                =  52;
  X_CreatePixmap               =  53;
  X_FreePixmap                 =  54;
  X_CreateGC                   =  55;
  X_ChangeGC                   =  56;
  X_CopyGC                     =  57;
  X_SetDashes                  =  58;
  X_SetClipRectangles          =  59;
  X_FreeGC                     =  60;
  X_ClearArea                  =  61;
  X_CopyArea                   =  62;
  X_CopyPlane                  =  63;
  X_PolyPoint                  =  64;
  X_PolyLine                   =  65;
  X_PolySegment                =  66;
  X_PolyRectangle              =  67;
  X_PolyArc                    =  68;
  X_FillPoly                   =  69;
  X_PolyFillRectangle          =  70;
  X_PolyFillArc                =  71;
  X_PutImage                   =  72;
  X_GetImage                   =  73;
  X_PolyText8                  =  74;
  X_PolyText16                 =  75;
  X_ImageText8                 =  76;
  X_ImageText16                =  77;
  X_CreateColormap             =  78;
  X_FreeColormap               =  79;
  X_CopyColormapAndFree        =  80;
  X_InstallColormap            =  81;
  X_UninstallColormap          =  82;
  X_ListInstalledColormaps     =  83;
  X_AllocColor                 =  84;
  X_AllocNamedColor            =  85;
  X_AllocColorCells            =  86;
  X_AllocColorPlanes           =  87;
  X_FreeColors                 =  88;
  X_StoreColors                =  89;
  X_StoreNamedColor            =  90;
  X_QueryColors                =  91;
  X_LookupColor                =  92;
  X_CreateCursor               =  93;
  X_CreateGlyphCursor          =  94;
  X_FreeCursor                 =  95;
  X_RecolorCursor              =  96;
  X_QueryBestSize              =  97;
  X_QueryExtension             =  98;
  X_ListExtensions             =  99;
  X_ChangeKeyboardMapping      = 100;
  X_GetKeyboardMapping         = 101;
  X_ChangeKeyboardControl      = 102;
  X_GetKeyboardControl         = 103;
  X_Bell                       = 104;
  X_ChangePointerControl       = 105;
  X_GetPointerControl          = 106;
  X_SetScreenSaver             = 107;
  X_GetScreenSaver             = 108;
  X_ChangeHosts                = 109;
  X_ListHosts                  = 110;
  X_SetAccessControl           = 111;
  X_SetCloseDownMode           = 112;
  X_KillClient                 = 113;
  X_RotateProperties	       = 114;
  X_ForceScreenSaver	       = 115;
  X_SetPointerMapping          = 116;
  X_GetPointerMapping          = 117;
  X_SetModifierMapping	       = 118;
  X_GetModifierMapping	       = 119;
  X_NoOperation                = 127;

implementation

end.
